﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Windows.Forms;

using ScintillaNET;
using System.Drawing;

using System.IO;

// deutsch

namespace html
{
    class Sci_Middleware
    {
        private static Sci_Middleware instance = new Sci_Middleware();

        //private static string StringBuilder sb = new StringBuilder();

        private Sci_Middleware()
        {
        }

        public static Sci_Middleware getInstance()
        {
            return instance;
        }

        public static void initSci(ref Scintilla scintilla)
        {
            scintilla.Dock = System.Windows.Forms.DockStyle.Fill;
            scintilla.LineWrapping.VisualFlags = ScintillaNET.LineWrappingVisualFlags.End;
            scintilla.Location = new System.Drawing.Point(0, 0);
            scintilla.Margins.Margin1.AutoToggleMarkerNumber = 0;
            scintilla.Margins.Margin1.IsClickable = true;
            scintilla.Margins.Margin2.Width = 16;
            scintilla.Name = "scintilla";
            scintilla.Size = new System.Drawing.Size(292, 266);
            scintilla.TabIndex = 0;            

            scintilla.UndoRedo.EmptyUndoBuffer();
            scintilla.Modified = false;

            scintilla.EndOfLine.IsVisible = false;
            scintilla.Margins.Margin0.Width = 40;

            //scintilla.Whitespace.Mode = WhitespaceMode.VisibleAlways;
            scintilla.Whitespace.Mode = WhitespaceMode.Invisible;

            //scintilla.LineWrapping.Mode = LineWrappingMode.Word;
            scintilla.LineWrapping.Mode = LineWrappingMode.None;

            scintilla.Zoom = 2;
            scintilla.Indentation.TabWidth = 3;
        }  // initsci


        public static void setLanguage_intern(ref Scintilla scintilla, string sFilename)
        {
            int i = sFilename.LastIndexOf(".");
            if (i > 0)
            {
                String sExt = sFilename.Substring(i + 1, sFilename.Length - i - 1).ToLower();
                String[] ext = { "asm", "asp", "bash", "batch", "cs", "cpp", "d", "ini", "lisp", "perl",  "css", "xml" };
                foreach (String s in ext)
                {
                    if (s.Equals(sExt))
                    {
                        scintilla.ConfigurationManager.Language = s;
                        return;
                    }
                }
                if (sExt.Equals("html") || sExt.Equals("xhtml") || sExt.Equals("htm") || sExt.Equals("php"))
                {
                    scintilla.ConfigurationManager.Language = "html";
                    return;
                }
                //if (sExt.Equals("php"))
                //{
                //    scintilla.ConfigurationManager.Language = "phpscript";
                //    return;
                //}
                
                if (sExt.Equals("pas"))
                {
                    scintilla.ConfigurationManager.Language = "pascal";
                    return;
                }
                if (sExt.Equals("sql"))
                {
                    scintilla.ConfigurationManager.Language = "mssql";
                    return;
                }
            }
        } // setLanguage_intern

        public static void setWhiteSpace(Scintilla scintilla, bool bWhitespace)
        {
            if (bWhitespace)
                scintilla.Whitespace.Mode = WhitespaceMode.VisibleAlways;
            else
                scintilla.Whitespace.Mode = WhitespaceMode.Invisible;

        }

        public static void setMnEndOfLine(Scintilla scintilla, bool bEndOfLine)
        {
            scintilla.EndOfLine.IsVisible = bEndOfLine;
        }



        public static void setLineNumbers(Scintilla scintilla, bool bLineNumbers)
        {
            if (bLineNumbers)
                scintilla.Margins.Margin0.Width = 35; // LINE_NUMBERS_MARGIN_WIDTH;
            else
                scintilla.Margins.Margin0.Width = 0;
        }


        public static void setWordWrap(Scintilla scintilla, bool bWordWrap)
        {
            if (bWordWrap)
                scintilla.LineWrapping.Mode = LineWrappingMode.Word;
            else
                scintilla.LineWrapping.Mode = LineWrappingMode.None;
        }


        public static void setZoom(Scintilla scintilla, int zoomLevel)
        {
            scintilla.Zoom = zoomLevel;
            if (zoomLevel < 10)
                scintilla.Margins.Margin0.Width = (int)(40 + zoomLevel / 2);
            else
                if (zoomLevel < 18)
                    scintilla.Margins.Margin0.Width = (int)(40 + (zoomLevel + 15) / 1.7);
                else
                    scintilla.Margins.Margin0.Width = (int)(40 + (zoomLevel + 25) / 1.7);
        }



        public static void search(Scintilla scintilla )
        {
            scintilla.FindReplace.ShowFind();
        }


        public static void searchreplace(Scintilla scintilla )
        {
            scintilla.FindReplace.ShowReplace();
        }

        public static void MakeUpper(Scintilla scintilla )
        {
            scintilla.Commands.Execute(BindableCommand.UpperCase);
        }

        public static void MakeLower(Scintilla scintilla )
        {
            scintilla.Commands.Execute(BindableCommand.LowerCase);
        }

        public static void CommentRange(Scintilla scintilla )
        {
            scintilla.Commands.Execute(BindableCommand.StreamComment);
        }

        public static void CommentLine(Scintilla scintilla )
        {
            scintilla.Commands.Execute(BindableCommand.LineComment);
        }
        public static void UnCommentLine(Scintilla scintilla )
        {
            scintilla.Commands.Execute(BindableCommand.LineUncomment);
        }


        public static void AutoComplete(Scintilla scintilla )
        {
            scintilla.AutoComplete.Show();
        }

        public static void InsertSnippet(Scintilla scintilla )
        {
            scintilla.Snippets.ShowSnippetList();
        }


        public static void SurroundWithSnippets(Scintilla scintilla )
        {
            scintilla.Snippets.ShowSurroundWithList();
        }

        public static void ShowGoto(Scintilla scintilla )
        {
            scintilla.GoTo.ShowGoToDialog();
        }




        public static void ToggleBookmarks(Scintilla scintilla )
        {
            Line currentLine = scintilla.Lines.Current;
            if (scintilla.Markers.GetMarkerMask(currentLine) == 0)
            {
                currentLine.AddMarker(0);
            }
            else
            {
                currentLine.DeleteMarker(0);
            }
        }


        public static void PreviousMarker(Scintilla scintilla )
        {
            //	 I've got to redo this whole FindNextMarker/FindPreviousMarker Scheme
            Line l = scintilla.Lines.Current.FindPreviousMarker(1);
            if (l != null)
                l.Goto();
        }
        public static void NextMarker(Scintilla scintilla )
        {
            //	 I've got to redo this whole FindNextMarker/FindPreviousMarker Scheme
            Line l = scintilla.Lines.Current.FindNextMarker(1);
            if (l != null)
                l.Goto();
        }
        public static void ClearMarker(Scintilla scintilla )
        {
            scintilla.Markers.DeleteAll(0);
        }


        public static void DropMarker(Scintilla scintilla )
        {
            scintilla.DropMarkers.Drop();
        }
        public static void CollectMarker(Scintilla scintilla )
        {
            scintilla.DropMarkers.Collect();
        }


        public static void undo(Scintilla scintilla )
        {
            scintilla.UndoRedo.Undo();
        }
        public static void redo(Scintilla scintilla )
        {
            scintilla.UndoRedo.Redo();
        }


        public static void cut(Scintilla scintilla )
        {
            scintilla.Clipboard.Cut();
        }
        public static void copy(Scintilla scintilla )
        {
            scintilla.Clipboard.Copy();
        }
        public static void paste(Scintilla scintilla )
        {
            scintilla.Clipboard.Paste();
        }
        public static void selectAll(Scintilla scintilla )
        {
            scintilla.Selection.SelectAll();
        }

        public static void PrintPreview(Scintilla scintilla )
        {
            scintilla.Printing.PrintPreview();
        }

        public static void Print(ref Scintilla scintilla )
        {
            scintilla.Printing.Print();
        }


        public static void FoldLevel(Scintilla scintilla )
        {
            scintilla.Lines.Current.FoldExpanded = true;
        }

        public static void UnfoldLevel(Scintilla scintilla )
        {
            scintilla.Lines.Current.FoldExpanded = false;
        }

        public static void FoldAll(Scintilla scintilla )
        {
            foreach (Line l in scintilla.Lines)
            {
                l.FoldExpanded = true;
            }
        }

        public static void UnfoldAll(Scintilla scintilla )
        {
            foreach (Line l in scintilla.Lines)
            {
                l.FoldExpanded = false;
            }
        }

        public static void NavigateForward(Scintilla scintilla )
        {
            scintilla.DocumentNavigation.NavigateForward();
        }

        public static void NavigateBackward(Scintilla scintilla )
        {
            scintilla.DocumentNavigation.NavigateBackward();
        }


        // ------------------------
        private static void InsertZeile(Scintilla scintilla, string sText)
        {
            Line currentLine = scintilla.Lines.Current;
            int position = currentLine.StartPosition;
            scintilla.InsertText(position, sText + "\r\n");
            scintilla.Modified = true;
        }

        private static void InsertText(Scintilla scintilla, string sText)
        {                       
            int position = scintilla.CurrentPos;
            scintilla.InsertText(position, sText);
            scintilla.Modified = true;
        }


        public static void InsertCSS_Body(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();            
            sb.AppendLine("body {");
            sb.AppendLine("  font-size: 1.2em;");
            sb.AppendLine("  color: rgb(100,100,100);");
            sb.AppendLine("  font-family: Arial, Helvetica, Verdana, sans-serif;");
            sb.AppendLine("  font-weight: 400;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_xx(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
              sb.AppendLine("xx {");
              sb.AppendLine("  font-family : Arial, Helvetica, Verdana, sans-serif;");
              sb.AppendLine("  font-size: 1.4em;");
              sb.AppendLine("  color: #0000FF;");
              sb.AppendLine("  font-weight : normal;");
              sb.AppendLine("}");
              InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_H2(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("h2{");
            sb.AppendLine("  font-size: 1.2em;");
            sb.AppendLine("  color: #0000FF;");
            sb.AppendLine("  font-weight : ligther;");
            sb.AppendLine("  background-color: yellow;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

        // nur HTML
        public static void InsertCSS_OL(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("ol{");
            sb.AppendLine("  list-style-type:upper-alpha;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }
        public static void InsertCSS_UL(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("ol{");
            sb.AppendLine("  list-style-type:square;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }
        public static void InsertCSS_TH(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("th{");
            sb.AppendLine("  font-family : Tahoma, Geneva, Arial, Helvetica, sans-serif;");
            sb.AppendLine("  font-size: 1.5em;");
            sb.AppendLine("  color1: #0000FF;");
            sb.AppendLine("  font-weight : normal;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }
        public static void InsertCSS_TD(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("td{");
            sb.AppendLine("  font-family : Tahoma, Geneva, Arial, Helvetica, sans-serif;");
            sb.AppendLine("  font-size: 1.2em;");
            sb.AppendLine("  color1: #FF00FF;");
            sb.AppendLine("  font-weight : normal;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertHTML_Scrolling(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<div style=\"overflow: auto; width: 100px; height: 100px; \">");
            sb.AppendLine("    <!-- Inhalt ->");
            sb.AppendLine("</div>");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_PositionAbsolute(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  position: absolute;");
            sb.AppendLine("  top: 50px;");
            sb.AppendLine("  left: 5;");
            sb.AppendLine("  width: 250px;");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_PositionRelativ(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  position: relative;");
            sb.AppendLine("  top: 50px;");
            sb.AppendLine("  left: 5;");
            sb.AppendLine("  width: 250px;");
            InsertZeile(scintilla, sb.ToString() );
        }



        public static void InsertCSS_Multidefinition(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("h1, h2 {");
            sb.AppendLine("  color: #0000FF;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_SelektorenBehind(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("body > h2 {");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }



        public static void InsertCSS_SelektorenTopDown(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("h1 + h2 {");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }



        public static void InsertCSS_SelektorenTogether(Scintilla scintilla)
        {
              StringBuilder sb = new StringBuilder();
              sb.AppendLine("/*  <div id=\"left\">  */");
              sb.AppendLine("#left{");              
              sb.AppendLine("  float: left;");
              sb.AppendLine("  top: 0;");
              sb.AppendLine("  left: 0;");
              sb.AppendLine("  width: 11em;");
              sb.AppendLine("  height:1000px;");
              sb.AppendLine("  border: 1px solid black;");
              sb.AppendLine("}");
              sb.AppendLine("");

              sb.AppendLine("/*  <div id=\"right\">  */");
              sb.AppendLine("#right{");
              sb.AppendLine("  right:100%"); 
              sb.AppendLine("  height:1000px;");
              sb.AppendLine("  width: 300px;");
              sb.AppendLine("  padding-left: 10px;");
              sb.AppendLine("  margin-left: 11.2em;");
              sb.AppendLine("  border: 1px inset  black;");
              sb.AppendLine("}");
              sb.AppendLine("");
              InsertZeile(scintilla, sb.ToString() );
        }



        public static void InsertCSS_background_image(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  background-image=url(\"bild.jpg\");");            
            InsertZeile(scintilla, sb.ToString() );
        }



        public static void InsertCSS_Body_backgroundrepeat(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  background-repeat:no-repeat;  /* repeat-x  repeat-y repeat */");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_Bodybackgroundattachment(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  background-attachment:scroll;  /* scroll  fixed=bleibt beim scrollen fixed */");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_Body_backgroundposition(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  background-position:left;  /* left center right top bottom,   w% x% y% z%, 0.3em */");            
            InsertZeile(scintilla, sb.ToString() );
        }






        public static void InsertCSS_ColorConstant(Scintilla scintilla)
        {
            FSelectColorConstant colorDlg = new FSelectColorConstant();
            colorDlg.ShowDialog();
            if (!colorDlg.SelectedColor.Equals("") )
            {
                string sColor = colorDlg.SelectedColor;
                InsertText(scintilla, sColor);
            }
        }



        public static void InsertCSS_BackgroundColor(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  background-color: gray;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        // Font

        public static void InsertCSS_FontSize_12em(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("    font-size: 1.2em;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_FontColor(Scintilla scintilla)
        {
            ColorDialog colorDlg = new ColorDialog();
            colorDlg.AnyColor = true;
            colorDlg.SolidColorOnly = false;
            if (colorDlg.ShowDialog() == DialogResult.OK)
            {
                string sColor = Basis.getHex(colorDlg.Color.R) + Basis.getHex(colorDlg.Color.G) + Basis.getHex(colorDlg.Color.B);
                InsertText(scintilla, "color:#" + sColor+";");
            }
        }


        public static void InsertCSS_Fonttyp_Verdana(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-family: VERDANA,ARIAL,HELVETICA;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_Fonttyp_CourierNew(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-family: COURIER NEW;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_Fonttyp_Monospace(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-family: MONOSPACE;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_SmallestFont(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-size: xx-small;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_SmallerFont(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-size: -1;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BiggerFont(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-size: +1;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BigFont(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-size: xx-large;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_FontSize_15Em(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-size: 1.5em;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_Bigger_Percentage(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-size: 150%;");            
            InsertZeile(scintilla, sb.ToString() );
        }




        public static void InsertCSS_FontWeight(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-weight:bold;  /* 100 bis 900 */");            
            InsertZeile(scintilla, sb.ToString() );
        }



        public static void InsertCSS_WordSpacing(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  word-spacing:2pt;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_LetterSpacing(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  letter-spacing:3pt;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_Textdecoration_None(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-decoration: none;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_Textdecoration_Underline(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-decoration: underline;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Textdecoration_Overline(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-decoration: overline;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Textdecoration_linethrough(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("   text-decoration: line-through;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Textdecoration_blink(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("   text-decoration:blink");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Textdecoration_Italic(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  font-style:italic;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_TextTransform_none(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-transform: none;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_TextTransform_uppercase(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-transform: uppercase;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_TextTransform_lowercase(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-transform: lowercase;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_TextTransform_capitalize(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-transform: capitalize;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Text_Indent(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-indent: 1.0in;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Valign_baseline(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  vertical-align: baseline;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Valign_Sub(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  vertical-align:sub;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Valign_Super(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  vertical-align:super;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Valign_texttop(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  vertical-align:text-top;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Valign_textbottom(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  vertical-align:text-bottom;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Valign_middle(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  vertical-align:middle;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Valign_top(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  vertical-align:top;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Valign_bottom(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  vertical-align:bottom;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Valign_percentage(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  vertical-align: 20%;  /*  -20%  oder +20% */");            
            InsertZeile(scintilla, sb.ToString() );
        }



        public static void InsertCSS_Textalign_left(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-align: left;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Textalign_center(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-align:center;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Textalign_right(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-align:right;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_Textalign_justify(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  text-align:justify; /* Blocksatz */");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_UL_listeNone(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: none;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_UL_Listecircle(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: circle;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_UL_Listesquare(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: square;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_UL_Listedisc(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: disc;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_OL_ListeNone(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: none;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_OL_Listedecimal(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: decimal;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_OL_Listedecimalleadingzero(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: decimal-leading-zero;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_OL_Listelowerroman(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: lower-roman;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_OL_Listeupperroman(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: upper-roman;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_OL_Listelowergreek(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: lower-greek;");            
            InsertZeile(scintilla, sb.ToString() );
        }
        public static void InsertCSS_OL_Listeuppergreek(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: upper-greek;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_OL_Listelowerlatin(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: lower-latin;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_OL_ListeUpperlatin(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-type: upper-latin;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_ListStylePosition_inside(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-position: inside;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_ListStylePosition_outside(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  list-style-position: outside;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_BlockTH(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("th {");
            sb.AppendLine("  font-family : Arial, Helvetica, Verdana, sans-serif;");
            sb.AppendLine("  font-size: 1.5em;");
            sb.AppendLine("  color1: #0000FF;");
            sb.AppendLine("  font-weight : normal;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_BlockTD(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("td {");
            sb.AppendLine("  font-family : Arial, Helvetica, Verdana, sans-serif;");
            sb.AppendLine("  font-size: 1.1em;");
            sb.AppendLine("  color1: #FF00FF;");
            sb.AppendLine("  font-weight : normal;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_TableBorderTDTHTable(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("/* Aussenrand */");
            sb.AppendLine("table {");
            sb.AppendLine("  border: 1px solid; ");
            sb.AppendLine("  border-collapse:collapse; /* verhindert doppelte Ränder */ ");
            sb.AppendLine("}");

            sb.AppendLine("");
            sb.AppendLine("/* einfacher Rand in der Tabelle */");
            sb.AppendLine("td, th {");
            sb.AppendLine("  border: 1px solid; ");
            sb.AppendLine("  border-color: black;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_FieldsetLegend(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("fieldset {");
            sb.AppendLine("  border: 2px solid #C5D8E1;");
            sb.AppendLine("  border-radius: 10px;");
            sb.AppendLine("  background: white;");
            sb.AppendLine("  margin: 0;");
            sb.AppendLine("  width:350px;");
            sb.AppendLine("}");
            sb.AppendLine("");
            sb.AppendLine("legend { ");
            sb.AppendLine(" color:red;");
            sb.AppendLine("} ");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_Bordercolor(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-color: red;");            
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertCSS_BorderStyleNone(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style: none;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderStyleHidden(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style: hidden;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderStyleDotted(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style: dotted;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderStyleDashed(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style:dashed;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderStyleSolid(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style:solid;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderStyleDouble(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style: double;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderStyleGroove(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style:groove;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderStyleRidge(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style:ridge;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderStyleInset(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style:inset;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderStyleOutset(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-style:outset;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_BorderWidth(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  border-width:5px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_PaddingOneForfour(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  padding: 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_PaddingTwoForTwo(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  padding: 10px 10px;  /* T/B  und L/R */");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_PaddingFourForFour(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  padding: 10px 10px 10px 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_PaddingTop(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  padding-top: 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_PaddingLeft(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  padding-left: 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_PaddingBottom(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  padding-bottom: 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_Paddingright(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  padding-right: 10px;;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_MarginOneForFour(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  margin: 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_MarginTwoForTwo(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  margin: 10px 10px;  /* oder 10px  auto;  */");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_MarginFourForFour(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  margin: 10px 10px 10px 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_MarginTop(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  margin-top: 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_MarginLeft(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  margin-left: 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_MarginBottom(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  margin-bottom: 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertCSS_MarginRight(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  margin-right: 10px;");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        


            
        public static void InsertCSS_Links_NoneVisited(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("a:link{");
            sb.AppendLine("  color: #0000FF;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

            
        public static void InsertCSS_Links_Visited(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("a:visited{");
            sb.AppendLine("  color: #00FF00;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

            
        public static void InsertCSS_Links_Activate(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("a:active{");
            sb.AppendLine("  color: #FF0000;");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

            
        public static void InsertCSS_Links_ThreeLinks(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("a:link{");            
            sb.AppendLine("    color: #0000FF;");            
            sb.AppendLine("}");            
            sb.AppendLine("");            
            sb.AppendLine("a:visited{");            
            sb.AppendLine("color: #00FF00;");            
            sb.AppendLine("}");            
            sb.AppendLine("");            
            sb.AppendLine("a:active{");            
            sb.AppendLine("  color: #FF0000;");            
            sb.AppendLine("}");            
            sb.AppendLine("");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }

            
        public static void InsertCSS_Links_ThreeLinksTogether(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("a:link, a:visited, a:active{");
            sb.AppendLine("  color: #FF0000;");
            sb.AppendLine("   /*text-decoration:none;*/");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

 //   HTML
        public static void InsertHTML_HTML5_strict11(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            // sb.AppendLine("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            sb.AppendLine("<!DOCTYPE html");
            sb.AppendLine("<html xmlns=\"http://www.w3.org/1999/xhtml\" >");
            sb.AppendLine("  <head>");
            sb.AppendLine("    <title> Beispiel 1 </title>");
            sb.AppendLine("      <meta name=\"author\" content=\"Administrator\"/>");
            sb.AppendLine("      <meta charset=\"utf-8\"/>");
            sb.AppendLine("  </head>");
            sb.AppendLine("<body>");  
            sb.AppendLine("    <h1>");
            sb.AppendLine("      Überschrift");
            sb.AppendLine("    </h1>");
            sb.AppendLine("  <section>");
            sb.AppendLine("    <header>");
            sb.AppendLine("");
            sb.AppendLine("    </header>");
            sb.AppendLine("");
            sb.AppendLine("");
            sb.AppendLine("    <footer>");
            sb.AppendLine("");
            sb.AppendLine("    </footer>");
            sb.AppendLine("  </section>");
            sb.AppendLine("</body>");
            sb.AppendLine("</html>");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertHTML_Basic_XHTML_strict11(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            sb.AppendLine("<!DOCTYPE html");
            sb.AppendLine("     PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"");
            sb.AppendLine("    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");
            sb.AppendLine("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">");
            sb.AppendLine("  <head>");
            sb.AppendLine("    <title> Beispiel 1 </title>");
            sb.AppendLine("      <meta name=\"author\" content=\"Administrator\"/>");
            sb.AppendLine("      <meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\"/>");
            sb.AppendLine("  </head>");
            sb.AppendLine("  <body>");  //  xmlns="http://www.w3.org/1999/xhtml"
            sb.AppendLine("    <h1>");
            sb.AppendLine("      Überschrift");
            sb.AppendLine("    </h1>");
            sb.AppendLine("  </body>");
            sb.AppendLine("</html>");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertHTML_Basic_XHTML(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<?xml version=\"1.0\" ?>");
            sb.AppendLine("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"");
            sb.AppendLine("        \"http://www.w3.org/TR/html4/strict.dtd\">");
            sb.AppendLine("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">");
            sb.AppendLine("  <head>");
            sb.AppendLine("    <title> Beispiel 1 </title>");
            sb.AppendLine("      <meta name=\"author\" content=\"Administrator\"/>");
            sb.AppendLine("      <meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\"/>");
            sb.AppendLine("  </head>");
            sb.AppendLine("  <body>");
            sb.AppendLine("    <h1>");
            sb.AppendLine("      Überschrift");
            sb.AppendLine("    </h1>");
            sb.AppendLine("  </body>");
            sb.AppendLine("</html>");
            InsertZeile(scintilla, sb.ToString() );
        }
        public static void InsertHTML_HTML4(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"");
            sb.AppendLine("          \"http://www.w3.org/TR/html4/loose.dtd\">");
            sb.AppendLine("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">");
            sb.AppendLine("  <head>");
            sb.AppendLine("    <title> Beispiel 1 </title>");
            sb.AppendLine("      <meta name=\"author\" content=\"Administrator\"/>");
            sb.AppendLine("      <meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\"/>");
            sb.AppendLine("  </head>");
            sb.AppendLine("  <body>");
            sb.AppendLine("    <h1>");
            sb.AppendLine("      Überschrift");
            sb.AppendLine("    </h1>");
            sb.AppendLine("  </body>");
            sb.AppendLine("</html>");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertHTML_Basic_Pur(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<html>");
            sb.AppendLine("  <head>");
            sb.AppendLine("    <title> Beispiel 1 </title>");
            sb.AppendLine("  </head>");
            sb.AppendLine("  <body>");
            sb.AppendLine("    <p>");
            sb.AppendLine("      1. Zeile");
            sb.AppendLine("    </p>");
            sb.AppendLine("  </body>");
            sb.AppendLine("</html>");
            InsertZeile(scintilla, sb.ToString() );
        }

                        
              
        public static void InsertHTML_Header(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <h1>");            
            sb.AppendLine("    Text H1");            
            sb.AppendLine("  </h1>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_Paragraph(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <p>");            
            sb.AppendLine("    Text p");            
            sb.AppendLine("  </p>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
             
        public static void InsertHTML_Header3(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <h3>");            
            sb.AppendLine("    Text H3");            
            sb.AppendLine("  </h3>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
             
        public static void InsertHTML_UL(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <ul>");            
            sb.AppendLine("    <li>Text1</li>");            
            sb.AppendLine("    <li>Text2</li>");
            sb.AppendLine("  </ul>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
        public static void InsertHTML_OL(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <ol>");            
            sb.AppendLine("    <li>Text1</li>");            
            sb.AppendLine("    <li>Text2</li>");
            sb.AppendLine("  </ol>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                      
        private static void createTable(Scintilla scintilla, int AnzRow, int AnzCol, 
                  bool bHeaderInFirstRow, bool bHeaderInFirstCol,
                  bool bHead, bool bFoot, bool bBody, bool bCellSpace_Padding, bool bMitColGroup,
                  int iCellFrame) {
            int nr, c, r;
            string sTable;
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<!-- Tabelle -->");
            sTable="  <table ";
            if (bCellSpace_Padding) sTable+=" cellspacing=\"20\" cellpadding=\"20%\" ";
            switch (iCellFrame) {
                case 1: sTable+="above ";
                        break;
                case 2: sTable+="below ";
                        break;
                case 3: sTable+="vsided ";
                        break;
                case 4: sTable+="lhs ";
                        break;
                case 5: sTable+="rhs ";
                        break;
                case 6: sTable+="box ";
                        break;
                case 7: sTable+="border ";
                        break;
            }
            sb.AppendLine(sTable+">");
            if (bMitColGroup) {
               sb.AppendLine("  <colgroup>");
               for (c=0; c<AnzCol; c++) {
                 sb.AppendLine("    <col width=\"80\"/>");
               }
               sb.AppendLine("  </colgroup>");
            }
                        
            if (bHead) {
                sb.AppendLine("   <thead>");
                sb.AppendLine("    <tr>");
                for (c = 0; c < AnzCol; c++)
                {
                    sb.AppendLine("     <th>");
                    sb.AppendLine("     </th>");
                }
                sb.AppendLine("    </tr>");
                sb.AppendLine("   </thead>");
            }

            nr=0;
            
            if (bBody) sb.AppendLine("   <tbody>");
            for (r=1; r<=AnzRow; r++) {                
                sb.AppendLine("     <tr>");
                for (c=1; c<=AnzCol; c++) {
                    nr++;
                    if ((c == 1) && bHeaderInFirstCol) {
                        if ((r == 1) && bHeaderInFirstCol) 
                            sb.AppendLine("      <th> <br /> </th>");
                        else 
                            sb.AppendLine("      <th> "+nr.ToString()+"-zelle</th>");
                    }
                    else {
                        if ((r == 1) && bHeaderInFirstRow) 
                            sb.AppendLine("      <th> <br /> </th>");
                        else 
                            sb.AppendLine("      <td> "+nr.ToString()+"-zelle</td>");
                    }
                }  // for (c=0; c<AnzCol; c++) {                  
                sb.AppendLine("     </tr>");
            }  // for (r=0; r<AnzRow; r++) {                
            if (bBody) sb.AppendLine("     </tbody>");

         
              if (bFoot) {
                  sb.AppendLine("   <tfoot>");
                sb.AppendLine("    <tr>");
                for (c = 0; c < AnzCol; c++)
                {
                    sb.AppendLine("     <td>");
                    sb.AppendLine("     </td>");
                }
                sb.AppendLine("    </tr>");
                sb.AppendLine("   </tfoot>");
              }
              sb.AppendLine("  </table>");
              InsertZeile(scintilla, sb.ToString() );
        }  // createTable
  
              
        public static void InsertHTML_Table(Scintilla scintilla)
        {
            FInsertHTMLTable form = new FInsertHTMLTable();
            form.ShowDialog();
            if (form.Retcode)
            {
                createTable(scintilla, form.AnzRow, form.AnzCol, form.HeaderInFirstRow, form.HeaderInFirstCol,
                    form.Head, form.Body, form.Foot, form.CellSpacing,  form.Colgroup,  form.CellFrame);
            }
        }
                        

        public static void InsertHTML_TR(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <tr>");            
            sb.AppendLine("    <td>Text1</td>");            
            sb.AppendLine("    <td>Text2</td>");
            sb.AppendLine("  </tr>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_HR(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <hr  />");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
                        
              
        public static void InsertHTML_EMailLink(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine( "  <a href=\"mailto:\"> Adresse </a>" );
            InsertText(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_Link(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <a href=\"bsp.html\">   text  </a>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
            
        public static void InsertHTML_Anchor(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine( "<a name=\"\"> </a>");
            InsertText(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_Comment(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <!--  Kommentar -->");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        



              
        public static void InsertHTML_Font_Bold(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<strong>  </strong>");            
            InsertText(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_Font_Italic(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<i>  </i>");            
            InsertText(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_Font_Underline(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<u>  </u>");            
            InsertText(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_Font_LineThrough(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<s>  </s>");            
            InsertText(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_Font_Sup(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<sup>  </sup>");            
            InsertText(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_Font_Sub(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<sub>  </sub>");            
            InsertText(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertHTML_Font_Char_Space(Scintilla scintilla)
        {            
            InsertText(scintilla, "&#x0020;");
        }
                        
        public static void InsertHTML_Font_Char_Tab(Scintilla scintilla)
        {
            InsertText(scintilla, "&#x0009;");
        }
                        
              
        public static void InsertHTML_Font_Char_CR(Scintilla scintilla)
        {
            InsertText(scintilla, "&#x000D;");
        }
                        
              
        public static void InsertHTML_Font_Char_LF(Scintilla scintilla)
        {
            InsertText(scintilla, "&#x000A;");
        }
                        
              
        public static void InsertHTML_Font_Char_Blank(Scintilla scintilla)
        {
            InsertText(scintilla, "&#160;");  // festes Leerzeichen
        }
                        
              
        public static void InsertHTML_Font_Char_Dash(Scintilla scintilla)
        {
            InsertText(scintilla, "&#173;");
        }
                        
              
        public static void InsertHTML_Font_Char_Copyright(Scintilla scintilla)
        {
            InsertText(scintilla, "&#169;");
        }
                        
              
        public static void InsertHTML_Font_Char_Copyright_R(Scintilla scintilla)
        {
            InsertText(scintilla, "&#174;");
        }
                        
              
        public static void InsertHTML_Font_Char_Trademark(Scintilla scintilla)
        {
            InsertText(scintilla, "&#x8482;");
        }
                        
              
        public static void InsertHTML_Font_Char_LT(Scintilla scintilla)
        {
            InsertText(scintilla, "&#60;");  // <
        }
                        
              
        public static void InsertHTML_Font_Char_GT(Scintilla scintilla)
        {
            InsertText(scintilla, "&#62;");
        }
                        
              
        public static void InsertHTML_Font_Char_GTGT(Scintilla scintilla)
        {
            InsertText(scintilla, "&#187;");
        }
                        
              
        public static void InsertHTML_Font_Char_LTLT(Scintilla scintilla)
        {
            InsertText(scintilla, "&#171;");
        }





        public static void InsertHTML_CSSReferenzExtern(Scintilla scintilla)
        {
            InsertText(scintilla, "&#171;");
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <!--  Muss in den Header-Abschnitt -->");            
            sb.AppendLine("<link rel=\"stylesheet\" type=\"text/css\"  ");
            sb.AppendLine("  href=\"style_bsp.css\" title=\"style1\"/>  ");            
            sb.AppendLine("  ");            
            InsertZeile(scintilla, sb.ToString() );
        }

    
              
        public static void InsertHTML_CSSReferenzLocal(Scintilla scintilla)
        {
            InsertText(scintilla, "&#171;");
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  <!--  Muss in den Header-Abschnitt -->  ");
            sb.AppendLine("  <style type=\"text/css\"  id=\"internalStyle\" >  ");            
            sb.AppendLine("    body {  ");            
            sb.AppendLine("      font-size: 1em;");            
            sb.AppendLine("      color: rgb(255,100,100);  ");            
            sb.AppendLine("      font-weight:300;  ");            
            sb.AppendLine("    }  ");            
            sb.AppendLine("  </style>  ");            
            sb.AppendLine("  ");            
            InsertZeile(scintilla, sb.ToString() );
        }




        public static void InsertHTML5_Header(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("header{");
            sb.AppendLine("   ");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertHTML5_Nav(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("nav{");
            sb.AppendLine("   ");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertHTML5_Section(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("section{");
            sb.AppendLine("   ");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertHTML5_Aside(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("aside{");
            sb.AppendLine("   ");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertHTML5_Footer(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("footer{");
            sb.AppendLine("   ");
            sb.AppendLine("}");
            InsertZeile(scintilla, sb.ToString() );
        }


        // --------------


        public static void InsertForm_Formrange1(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<form>");
            sb.AppendLine("");
            sb.AppendLine("</form>");
            InsertZeile(scintilla, sb.ToString() );
        }
        public static void InsertForm_Formrange2(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<form method=\"get\" action=\"http://mwilhelm.hs-harz.de/scripte/php/test.php\" >");
            sb.AppendLine("");
            sb.AppendLine("</form>");
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertForm_FieldsetLegend(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<fieldset>");            
            sb.AppendLine("  <legend>Überschrift</legend>");            
            sb.AppendLine("</fieldset>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertForm_Label(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<label> Username  ");            
            sb.AppendLine("  <!-- hier ein Formular-Element eintragen -->  ");            
            sb.AppendLine("</label>  ");            
            sb.AppendLine("<br />");            
            sb.AppendLine("<br />");            
            sb.AppendLine("  ");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertForm_Textfeld1(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<input type=\"text\" name=\"name1\" value=\"Vorgabetext\" size=\"12\" />");
            InsertZeile(scintilla, sb.ToString() );
        }
        public static void InsertForm_Textfeld2(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<input type=\"text\" name=\"name1\" placeholder=\"Vorgabetext(placeholder)\" value=\"Vorgabetext\" size=\"12\"   autofocus=\"autofocus\"  required=\"required\"/>");
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertForm_TextfeldPattern(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<!--  IP-Adresse  pattern=\"[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}\" --> ");
            sb.AppendLine("<!--  Name: pattern=\"[A-Z][a-z]{1,30}\" --> ");            
            sb.AppendLine("<input type=\"text\" name=\"name1\" placeholder=\"Vorgabetext(placeholder)\" pattern=\"[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}\" value=\"Vorgabetext\" size=\"12\"   autofocus=\"autofocus\"  required=\"required\"/>");            
            sb.AppendLine("  ");       
            InsertZeile(scintilla, sb.ToString() );
        }
                        
            
        public static void InsertForm_Textarea(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<br />");
            sb.AppendLine("<!-- Moegliche Varianten von warp: a)off b)HARD  c)SOFT -->");
            sb.AppendLine("<textarea name=\"editor\" rows=\"10\" cols=\"50\"  wrap=\"SOFT\"> Eingabe eines Textes in einem Editor</textarea>");
            InsertZeile(scintilla, sb.ToString() );
        }
                        
             
        public static void InsertForm_Hidden(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<input type=\"hidden\" name=\"geheim\" value=\"passwort\" size=\"12\"/>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertForm_CheckBox(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<input type=\"checkbox\" name=\"cad1\" value=\"CAD1\" /> AutoCAD<br />");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertForm_RadioButton(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<br />");    
            sb.AppendLine("<input type=\"radio\" name=\"R1\" value=\"CAD1\" checked=\"checked\" /> AutoCAD  <br />");    
            sb.AppendLine("<input type=\"radio\" name=\"R1\" value=\"CAD2\"  /> TurboCAD <br />");    
            sb.AppendLine("<input type=\"radio\" name=\"R1\" value=\"CAD3\"  /> Eagle<br />");    
            sb.AppendLine("");    
            InsertZeile(scintilla, sb.ToString() );
        }
                        
             
        public static void InsertForm_ComboBox(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<br />");            
            sb.AppendLine("<select name=\"objekte\" >");            
            sb.AppendLine("  <option > Kegel </option >");            
            sb.AppendLine("  <option selected=\"selected\" > Zylinder </option >");            
            sb.AppendLine("  <option  > Quader </option >");            
            sb.AppendLine("</select>");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
             
        public static void InsertForm_HTML5_Number(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("1. Zahl <input type=\"number\" name=\"zahl1\" value=\"zahl1\" /> <br />");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertForm_HTML5_Range(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<input type=\"range\" name=\"range1\" min=\"10\" max=\"100\" step=\"5\" value=\"55\" />");            
            sb.AppendLine("<output name=\"range1_output\" onforminput=\"value=range1.value\"> 55 </output> <br />");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertForm_HTML5_Color(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("Farbdialog: <input type=\"color\" name=\"color1\" value=\"color1\" /> <br />");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertForm_HTML5_Date(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine(" 1. Datum: <input type=\"date\" name=\"datum1\" value=\"datum1\" /><br />");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }
                     


        public static void InsertForm_HTML5_Meter(Scintilla scintilla)
        {
            InsertZeile(scintilla, "<meter id=\"meter1\" min=\"0\"  max=\"100\" value=\"25.67\" title=\"KW\" > </meter>");
        }

        
        public static void InsertForm_HTML5_Output(Scintilla scintilla)
        {
            InsertZeile(scintilla, "<output for=\"a b\" id=\"ergebnis\">   </output>");
        }

        
        public static void InsertForm_HTML5_Canvas(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine(" 1. Datum: <input type=\"date\" name=\"datum1\" value=\"datum1\" /><br />");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }



        public static void InsertForm_Submit(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<input type=\"submit\" value=\"senden\" />");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
              
        public static void InsertForm_Button(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<input type=\"button\" value=\"senden\" onclick=\"calc(this.form)\"/>");            
            InsertZeile(scintilla, sb.ToString() );
        }
                        
             
        public static void InsertForm_Reset(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<input type=\"reset\" value=\"L&#246;schen\"/><br />");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertForm_CompletSample(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendLine("<form>");
            sb.AppendLine("  <fieldset>");
            sb.AppendLine("   <legend>Überschrift</legend>");
            sb.AppendLine("   <label> Eingabe");
            sb.AppendLine("     <input type=\"text\" name=\"eingabe\"  placeholder=\"Vorgabetext(placeholder)\" value=\"Vorgabetext\" size=\"14\" maxlength=\"10\" autofocus=\"autofocus\"  required=\"required\" />");
            sb.AppendLine("   </label>");
            sb.AppendLine("   <br />");
            sb.AppendLine("   <br />");
            sb.AppendLine("   <input type=\"submit\" value=\"senden\" />");
            sb.AppendLine("  <input type=\"reset\" value=\"L&#246;schen\"/><br />");
            sb.AppendLine("  <br />");
            sb.AppendLine(" </fieldset>");
            sb.AppendLine("</form>");
            sb.AppendLine("");
            InsertZeile(scintilla, sb.ToString() );
        }



        public static void InsertForm_HTML5_Header(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<header>");
            sb.AppendLine("");
            sb.AppendLine("</header>");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertForm_HTML5_Arcticle(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<article>");
            sb.AppendLine("");
            sb.AppendLine("</article>");
            InsertZeile(scintilla, sb.ToString() );
        }
            

        public static void InsertForm_HTML5_Section(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<section>");
            sb.AppendLine("  <header>");
            sb.AppendLine("");
            sb.AppendLine("  </header>");
            sb.AppendLine("");
            sb.AppendLine("");
            sb.AppendLine("");
            sb.AppendLine("");
            sb.AppendLine("  <footer>");
            sb.AppendLine("");
            sb.AppendLine("  </footer>");
            sb.AppendLine("</section>");
            InsertZeile(scintilla, sb.ToString() );
        }
            

        public static void InsertForm_HTML5_Aside(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<h2>");
            sb.AppendLine("Ein besonderes Angebot");
            sb.AppendLine("</h2>");
            sb.AppendLine("<aside id=\"angebot1\">");
            sb.AppendLine(" <h2>Shop</h2>");
            sb.AppendLine("</aside>");
            InsertZeile(scintilla, sb.ToString() );
        }
            

        public static void InsertForm_HTML5_Footer(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<footer>");
            sb.AppendLine("");
            sb.AppendLine("</footer>");
            InsertZeile(scintilla, sb.ToString() );
        }
            

        public static void InsertForm_HTML5_Progess(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<progress id=\"progress1\">");
            sb.AppendLine("</progress>");
            InsertZeile(scintilla, sb.ToString() );
        }
            

        public static void InsertForm_HTML5_Datetime(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<datetime>");
            sb.AppendLine("");
            sb.AppendLine("</datetime>");
            InsertZeile(scintilla, sb.ToString() );
        }
            

        public static void InsertForm_HTML5_Address(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<address>");
            sb.AppendLine("");
            sb.AppendLine("</address>");
            InsertZeile(scintilla, sb.ToString() );
        }
            

        public static void InsertForm_HTML5_Summary(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<summary>");
            sb.AppendLine("");
            sb.AppendLine("</summary>");
            InsertZeile(scintilla, sb.ToString() );
        }
            

        public static void InsertForm_HTML5_Details(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<details>");
            sb.AppendLine("");
            sb.AppendLine("</details>");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertForm_HTML5_Video(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<video>");
            sb.AppendLine("  <source  src=\"video1.mp4\"  type=\"video/mp4\">");  // h.264
            sb.AppendLine("  <source  src=\"video1.ogv\"  type=\"video/ogg\">");  // mp
            sb.AppendLine("</video>");
            InsertZeile(scintilla, sb.ToString());
        }

        public static void InsertForm_HTML5_Audio(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<audio>");
            sb.AppendLine("  <source  src=\"audio1.mp3\"  type=\"audio/mpeg\" />");  // 
            sb.AppendLine("  <source  src=\"audio1.ogg\"  type=\"audio/ogg\" />");  // mp3
            sb.AppendLine("</audio>");
            InsertZeile(scintilla, sb.ToString());
        }

        public static void InsertForm_HTML5_Wbr(Scintilla scintilla) {
            InsertZeile(scintilla, "<wbr>");
        }
        public static void InsertForm_HTML5_S(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            InsertZeile(scintilla, "<s>    </s>");
        }
        public static void InsertForm_HTML5_Hr(Scintilla scintilla) {
            InsertZeile(scintilla, "<hr>");
        }
        public static void InsertForm_HTML5_Small(Scintilla scintilla) {
            InsertZeile(scintilla, "<small>  </small>");
        }

        public static void InsertForm_HTML5_Strong(Scintilla scintilla) {
            InsertZeile(scintilla, "<strong>  </strong>");
        }
        public static void InsertForm_HTML5_Em(Scintilla scintilla) {
            InsertZeile(scintilla, "<em>  </em>");
        }

        public static void InsertForm_HTML5_Abbr(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<abbr>");
            sb.AppendLine("");
            sb.AppendLine("</abbr>");
            InsertZeile(scintilla, sb.ToString());
        }


        public static void InsertForm_HTML5_Code(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<code>");
            sb.AppendLine("");
            sb.AppendLine("</code>");
            InsertZeile(scintilla, sb.ToString());
        }
        public static void InsertForm_HTML5_Var(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<var>");
            sb.AppendLine("");
            sb.AppendLine("</var>");
            InsertZeile(scintilla, sb.ToString());
        }
        public static void InsertForm_HTML5_Cite(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<cite>");
            sb.AppendLine("");
            sb.AppendLine("</cite>");
            InsertZeile(scintilla, sb.ToString());
        }
        
        public static void InsertForm_HTML5_Dl(Scintilla scintilla) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<dl>");
            sb.AppendLine(" <dt> Punkt1</dt>");
            sb.AppendLine(" <dt> Punkt2</dt>");
            sb.AppendLine("  <dd>");
            sb.AppendLine("   Text");
            sb.AppendLine("  </dd>");
            sb.AppendLine("</dl>");
            InsertZeile(scintilla, sb.ToString());
        }

       public static void InsertForm_HTML5_Dfn(Scintilla scintilla) {
            InsertZeile(scintilla, "<dfn>HTML</dfn> heisst HyperText Markup Language");
        }






              //Sci_Middleware.(scintilla);            
        public static void InsertJavaScript_InsertRange(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<!--  Muss in den Header-Abschnitt -->");            
            sb.AppendLine("    <script type=\"text/javascript\">");            
            sb.AppendLine("       // <![CDATA[");            
            sb.AppendLine("      \"use strict;\"");       
            sb.AppendLine("      function  f( form ) {");            
            sb.AppendLine("        \"use strict;\"");
            sb.AppendLine("         alert (\"hier in f\") ;");            
            sb.AppendLine("      }");       
            sb.AppendLine("      // ]]>");            
            sb.AppendLine("    </script>");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }

              
        public static void InsertJavaScript_Function(Scintilla scintilla)
        {
            FInput form = new FInput();
            form.ShowDialog();
            string sName = form.Retcode;
            if (!sName.Equals(""))
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("function "+sName+"() {");
                sb.AppendLine("  \"use strict;\"");
                sb.AppendLine("");
                sb.AppendLine("  return;");
                sb.AppendLine("}");
                sb.AppendLine("");
                InsertZeile(scintilla, sb.ToString() );
            }
        }

             
        public static void InsertJavaScript_Consolelog(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("console.log( \"hier beim Punkt1\" );");            
            InsertZeile(scintilla, sb.ToString() );
        }

             
        public static void InsertJavaScript_Event_onload(Scintilla scintilla)
        {
            InsertText(scintilla, "onLoad=\"\"");
        }

             
        public static void InsertJavaScript_Event_onUnload(Scintilla scintilla)
        {
            InsertText(scintilla, "onUnload=\"\"");
        }

             
        public static void InsertJavaScript_Event_onClick(Scintilla scintilla)
        {
            InsertText(scintilla, "onClick=\"\"");
        }

             
        public static void InsertJavaScript_onDblclick(Scintilla scintilla)
        {
            InsertText(scintilla, "onDBLclick=\"\"");
        }

             
        public static void InsertJavaScript_Event_onMouseDown(Scintilla scintilla)
        {
            InsertText(scintilla, "onMouseDown=\"\"");
        }

            
        public static void InsertJavaScript_Event_onMouseUp(Scintilla scintilla)
        {
            InsertText(scintilla, "onMouseUp=\"\"");
        }

            
        public static void InsertJavaScript_Event_onMouseOver(Scintilla scintilla)
        {
            InsertText(scintilla, "onMouseOver=\"\"");
        }

            
        public static void InsertJavaScript_Event_onMouseMove(Scintilla scintilla)
        {
            InsertText(scintilla, "onMouseMoveonLoad=\"\"");
        }

            
        public static void InsertJavaScript_Event_onMouseOut(Scintilla scintilla)
        {
            InsertText(scintilla, "onMouseOut=\"\"");
        }

            
        public static void InsertJavaScript_Event_onFocus(Scintilla scintilla)
        {
            InsertText(scintilla, "onFocus=\"\"");
        }

            
        public static void InsertJavaScript_Event_onBlur(Scintilla scintilla)
        {
            InsertText(scintilla, "onBlur=\"\"");
        }

            
        public static void InsertJavaScript_Event_onKeypress(Scintilla scintilla)
        {
            InsertText(scintilla, "onKeypress=\"\"");
        }

            
        public static void InsertJavaScript_Event_onKeydown(Scintilla scintilla)
        {
            InsertText(scintilla, "onKeydown=\"\"");
        }

            
        public static void InsertJavaScript_Event_onKeypup(Scintilla scintilla)
        {
            InsertText(scintilla, "onKeyup=\"\"");
        }

            
        public static void InsertJavaScript_Event_onSubmit(Scintilla scintilla)
        {
            InsertText(scintilla, "onSubmit=\"\"");
        }

            
        public static void InsertJavaScript_Event_onReset(Scintilla scintilla)
        {
            InsertText(scintilla, "onReset=\"\"");
        }

            
        public static void InsertJavaScript_Event_onSelect(Scintilla scintilla)
        {
            InsertText(scintilla, "onSelect=\"\"");
        }

            
        public static void InsertJavaScript_Event_onChange(Scintilla scintilla)
        {
            InsertText(scintilla, "onChange=\"\"");
        }

            
        public static void InsertJavaScript_If(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("if() {");            
            sb.AppendLine("");            
            sb.AppendLine("}");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }

            
        public static void InsertJavaScript_If_Else(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("if() {");
            sb.AppendLine("");
            sb.AppendLine("}");
            sb.AppendLine("else {");            
            sb.AppendLine("");            
            sb.AppendLine("}");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }

            
        public static void InsertJavaScript_Dowhile(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("do {");
            sb.AppendLine("");
            sb.AppendLine("} while();");
            InsertZeile(scintilla, sb.ToString() );
        }

            
        public static void InsertJavaScript_While(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("while() {");
            sb.AppendLine("");
            sb.AppendLine("}");
            sb.AppendLine("");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertJavaScript_For(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("for ( ; ; ) {");            
            sb.AppendLine("");            
            sb.AppendLine("}");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertJavaScript_For_i(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("for (i=; i< ; i++ ) {");            
            sb.AppendLine("");            
            sb.AppendLine("}");            
            sb.AppendLine("");            
            InsertZeile(scintilla, sb.ToString() );
        }
 

        public static void InsertJavaScript_Switch(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("switch () {");
            sb.AppendLine("  case 0:");
            sb.AppendLine("         break;");
            sb.AppendLine("  case 1:");
            sb.AppendLine("         break;");
            sb.AppendLine("  default:");
            sb.AppendLine("         break;");
            sb.AppendLine("}");
            sb.AppendLine("");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertJavaScript_ParseInt(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("var n = parseInt(sn);");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertJavaScript_ParseFloat(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("var n = parseInt(sn,10);");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertJavaScript_isNaN(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("if ( isNaN(sn) ) {");
            sb.AppendLine("");
            sb.AppendLine("}");
            sb.AppendLine("else {");
            sb.AppendLine("");
            sb.AppendLine("}");
            sb.AppendLine("");
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertJavaScript_komma2Point(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("sn = sn.replace( /,/,\".\" );");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertJavaScript_toFixed(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("var erg = x.toFixed(3);");            
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertJavaScript_split(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("var musik = \"Abba;Beatles;Zappa\";");            
            sb.AppendLine("var values = musik.split(\";\")");            
            sb.AppendLine("var erg1 = values[0];");            
            sb.AppendLine("var erg2 = values[1];");         
            sb.AppendLine("var erg3 = values[2];");            
            sb.AppendLine("");         
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void InsertJavaScript_Popupwindow(Scintilla scintilla)
        {
           // error
        }



              
        public static void InsertPHP_Range(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("<?php");
            sb.AppendLine("");
            sb.AppendLine("");
            sb.AppendLine("?>");           
            InsertZeile(scintilla, sb.ToString() );
        }


        public static void PHP_InsertPHP_isset(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("if ( isset($_GET[''vorname'']) ){		 //	Ist gesetzt?");
            sb.AppendLine("  $vorname = $_GET[''vorname''];");
            sb.AppendLine("}");                        
            InsertZeile(scintilla, sb.ToString() );
        }
              
        public static void InsertPHP_Isnumeric(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  if (is_numeric($sZahl)) {");
            sb.AppendLine("     $Zahl = intval ( $sZahl, 10 );");
            sb.AppendLine("  } else {");
            sb.AppendLine("     echo \"Konvertierungsfehler: Die Variable '???' ist keine Zahl\", PHP_EOL;");
            sb.AppendLine("  }");
            InsertZeile(scintilla, sb.ToString() );
        }

              
        public static void InsertPHP_IntVal(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  $Zahl = intval ( $sZahl );");
            InsertZeile(scintilla, sb.ToString() );
        }

              
        public static void InsertPHP_echoText(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  echo \"<p>Hier kommt ein Text in der HTML-Datei</p>\",PHP_EOL;");
            InsertZeile(scintilla, sb.ToString() );
        }

              
        public static void InsertPHP_echoStringAndVariable(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  $Zahl=42;");
            sb.AppendLine("  echo \"<p>Zahl: $Zahl</p>\r\n\",PHP_EOL;");
            InsertZeile(scintilla, sb.ToString() );
        }

              
        public static void InsertPHP_echoStringAndVariable_withPoint(Scintilla scintilla)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("  $Zahl=42;");
            sb.AppendLine("  echo \"<p>Zahl: \" . $Zahl . \"</p>\",PHP_EOL;");
            InsertZeile(scintilla, sb.ToString() );
        }

        public static void InsertHTML_InsertImage(Scintilla scintilla, string sImagefile, bool bWithCoord)
        {
            StringBuilder sb = new StringBuilder();
            if (bWithCoord) 
            {
                sb.AppendLine("  <img src=\"" + Basis.getFilename(sImagefile) + "\" alt=\"Beschreibender Text\" border=\"0\" usemap=\"#shapeareaname\" />");
                sb.AppendLine("  <map name=\"shapeareaname\">");
                sb.AppendLine("  ");
                sb.AppendLine("  </map>");
            }
            else {
                sb.AppendLine("  <img src=\"" + Basis.getFilename(sImagefile) + "\" alt=\"Beschreibender Text\" border=\"0\" />");
            }
            InsertZeile(scintilla, sb.ToString() );
        }
            

        public static void InsertImageCoords(Scintilla scintilla)
        {
             FCoords dialog = new FCoords();
             dialog.ShowDialog();
             string sCoords;
             if (dialog.RetCode)
             {
                sCoords = dialog.getCoords();
                StringBuilder sb = new StringBuilder();
                sb.AppendLine("  <br />");
                sb.AppendLine("  "+sCoords);
                sb.AppendLine("  <br />");
                InsertZeile(scintilla, sb.ToString() );
             }
         }

        public static void InsertImageCoordsFromSHWFile(Scintilla scintilla)
        {
            int i, j, Image_w, Image_h;
            int m, n;
            double minX = 0.0, maxX = 0.0;
            double minY = 0.0, maxY = 0.0;
            double Faktor, x, y, yPixel1, yPixel2;
            int ix, iy;
            int diff_x, diff_y;
            string sName;
            string sKoord;            
            int anz;
            PictureBox pictureBox1;
            string sLine;

              OpenFileDialog openFileDialog = new OpenFileDialog();
              openFileDialog.DefaultExt = ".bmp";
              openFileDialog.InitialDirectory = System.Environment.CurrentDirectory; //  Environment.GetFolderPath(Environment.SpecialFolder.Personal);
              openFileDialog.Filter =   "JPG-Dateien (*.jpg;*.jpeg)|*.jpg;*.jpeg|" + 
                                        "BMP-Dateien (*.bmp)|*.bmp|" +
                                        "PING-Dateien (*.png)|*.png|" +
                                        "GIF-Dateien (*.gif)|*.gif|" +
                                        "Alle Dateien (*.*)|*.*";

              openFileDialog.Multiselect = false;
              openFileDialog.Title = "Bildatei Öffnen";

              if (openFileDialog.ShowDialog(null) == DialogResult.OK)
              {
                  pictureBox1 = new System.Windows.Forms.PictureBox();
                  pictureBox1.Image = System.Drawing.Image.FromFile(openFileDialog.FileName);
                  pictureBox1.SizeMode = PictureBoxSizeMode.Normal;
                  Image_w = pictureBox1.Image.Width;
                  Image_h = pictureBox1.Image.Height;
                                    
                  openFileDialog.Filter = "SHW-Dateien (*.shw)|*.shw|" + "Alle Dateien (*.*)|*.*";
                  openFileDialog.Title = "Koordinatendatei Öffnen";
                  openFileDialog.DefaultExt = ".shw";
                  openFileDialog.FileName = "*.shw";
                  if (openFileDialog.ShowDialog(null) == DialogResult.OK)
                  {
                     string sFilename = openFileDialog.FileName;
                     FileStream inFile = new FileStream(sFilename, FileMode.Open);
                     StreamReader inStream = new StreamReader(inFile);
                     sLine = inStream.ReadLine();  // readln(kanal,n);  // Typ
                     n = Convert.ToInt32(sLine);
                     if ((n != 2) && (n != 3))
                     {
                         MessageBox.Show("Die SHW-Datei hat ein falsches Format (Punktshape)", "Hinweis", MessageBoxButtons.OK, MessageBoxIcon.Error);
                         inStream.Close();
                         inFile.Close();
                         return;
                     }

                      // Bestimme min und max
                     sLine = inStream.ReadLine();  // Anzahl der Linien
                     n = Convert.ToInt32(sLine);
                     for (i=0; i<n; i++) {
                        sLine = inStream.ReadLine();  // Name
                        sLine = inStream.ReadLine();  // Anzahl der Koord
                        m = Convert.ToInt32(sLine);
                        for (j=0; j<m; j++) {
                            sLine = inStream.ReadLine();  // Anzahl der Koord
                            x = Convert.ToDouble(sLine, System.Globalization.CultureInfo.InvariantCulture);
                            sLine = inStream.ReadLine();  // Anzahl der Koord
                            y = Convert.ToDouble(sLine, System.Globalization.CultureInfo.InvariantCulture);
                            if ((i==1) && (j==1) ) {
                                minX = x;
                                maxX = x;
                                minY = y;
                                maxY = y;
                            } else {
                             if (x<minX) minX = x;
                             if (x>maxX) maxX = x;
                             if (y<minY) minY = y;
                             if (y>maxY) maxY = y;
                            }
                        } // for j
                     } // for i
                     inStream.Close();
                     inFile.Close();
                     diff_x = (int) Math.Round(maxX-minX);
                     diff_y = (int) Math.Round(maxY - minY);
                     x = diff_x / Image_w;
                     y = diff_y / Image_h;
                    if (x>y)
                        Faktor=x;
                    else
                        Faktor=y;


                    inFile = new FileStream(sFilename, FileMode.Open);
                    inStream = new StreamReader(inFile);
                    sLine = inStream.ReadLine();  // readln(kanal,n);  // Typ
                    n = Convert.ToInt32(sLine);
                    if ((n != 2) && (n != 3))
                    {
                        MessageBox.Show("Die SHW-Datei hat ein falsches Format (Punktshape)", "Hinweis", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        inStream.Close();
                        inFile.Close();
                        return;
                    }

                    StringBuilder sb = new StringBuilder();                    
                    sLine = inStream.ReadLine();  // Anzahl der Linien
                    n = Convert.ToInt32(sLine);
                    for (i = 0; i < n; i++)
                    {
                        sName = inStream.ReadLine();  // Name                        
                        sLine = inStream.ReadLine();  // Anzahl der Koord
                        m = Convert.ToInt32(sLine);
                        sb.AppendLine("    <area shape=\"poly\" alt=\"" + sName + "\" coords=");
                        sKoord="       \"";
                        anz=0;
                        for (j = 0; j < m; j++)
                        {
                            sLine = inStream.ReadLine();  // Anzahl der Koord
                            x = Convert.ToDouble(sLine, System.Globalization.CultureInfo.InvariantCulture);
                            sLine = inStream.ReadLine();  // Anzahl der Koord
                            y = Convert.ToDouble(sLine, System.Globalization.CultureInfo.InvariantCulture);
                           ix = (int) Math.Round( (x-minX)/Faktor);
                            // Y-Achse ist in ArcView vertauscht, Ymax ist oben
                            // Formel:  y(pixel) =   (1.0 - ((Yi-Ymin)/(Ymax-Ymin))*ImageHeight
                           yPixel1 = (y - minY) / (maxY - minY);  // pronzetualer Anteil in der Originalhoehe
                           yPixel2 = (1.0 - yPixel1);
                           iy = (int)Math.Round(yPixel2 * Image_h);
                           if (j+1<m)
                             sKoord += ix.ToString()+","+iy.ToString()+" ,";
                           else
                             sKoord += ix.ToString()+","+iy.ToString();
                           anz++;
                           if (anz == 10)
                           {
                               sb.AppendLine(sKoord);
                               sKoord = "       ";
                               anz = 0;
                           }
                            
                        } // for j
                        sKoord += "\" href=\"gem_"+(i+1).ToString()+".xhtml\" />";
                        sb.AppendLine(sKoord);
                    } // for i
                    inStream.Close();
                    inFile.Close();
                    InsertZeile(scintilla, sb.ToString() );
                  }  // if openDialog
             }
        }


    }
}
